// ************************************************************************
// ***************** Copyright by Micha Kurzela @ 2013 *******************
// ******************** Firmware do kitu AVT5341 **************************
// ************************************************************************


#include <avr/io.h>
#include <util/delay.h>

#define ZAL_TRAFO PORTB|=0b00000001
#define WYL_TRAFO PORTB&=0b11111110

#define ZAL_PK1 PORTD|=0b00000001
#define WYL_PK1 PORTD&=0b11111110

#define ZAL_PK2 PORTD|=0b00000010
#define WYL_PK2 PORTD&=0b11111101

#define ZAL_PK1_DOD PORTD|=0b00000100
#define WYL_PK1_DOD PORTD&=0b11111011

#define ZAL_PK2_DOD PORTD|=0b00001000
#define WYL_PK2_DOD PORTD&=0b11110111

#define PRZYC1 bit_is_clear(PINB, PINB7) //jego wcinicie = otwieranie
#define PRZYC2 bit_is_clear(PINB, PINB6) //zamykanie

#define JEST_REZ_DOD bit_is_clear(PINC, PINC0) //wykrycie pytki przystawki

#define POT_MIN ADMUX=0b00000011;
#define POT_MAX ADMUX=0b00000100;
#define REZ ADMUX=0b00000101;
#define POT_MIN_DOD ADMUX=0b00000001;
#define POT_MAX_DOD ADMUX=0b00000010;
#define REZ_DOD ADMUX=0b00000000;

#define IL_CYKLI 50000 //ilo cykli do samozamknicia dla bramy jednoskrzydowej
#define IL_CYKLI_DOD 25000 //ilo cykli do samozamknicia dla bramy dwuskrzydowej
//jeden cykl trwa ok. 6ms
//w trybie dwuskrzydowym ok. 12ms

#define OPOZN_ZAL_TRAFA 100 //ilo milisekund midzy uruchomieniem transformatora a wczeniem siownika
#define OPOZN_M_SILOWNIKAMI 50 //ilo milisekund midzy uruchomieniem jednego siownika a startem drugiego
#define OPOZN_ROZRUCH 100 //czas "niewraliwoci" ukadu na przecienie/niedocienie siownika w milisekundach


uint8_t tryb_pracy; //0 - brama jednoskrzydowa; 1 - brama dwuskrzydowa
uint16_t wart_rez, wart_pot_min, wart_pot_max; //wartoci z ADC: rezystora, potencjometra MIN, potencjometra MAX
uint16_t wart_rez_dod, wart_pot_min_dod, wart_pot_max_dod; //wartoci z ADC: rezystora, potencjometra MIN, potencjometra MAX dla pytki przystawki
uint8_t otw=0, zam=0, autozam=0; //flagi: otwierania, zamykania i autozamykania
uint8_t otw_dod=0, zam_dod=0; //flagi: otwierania i zamykania dla drugiego skrzyda

uint16_t odliczanie; //zmienna odliczajca do samozamknicia
uint8_t poprz_przyc1=1, poprz_przyc2=1; //zmienne do przechowywania "poprzednich" stanw przyciskw
uint8_t akt_przyc1=1, akt_przyc2=1; //zmienne do przechowywania "aktualnych" stanw przyciskw
// 0 - przycisk wcinity ; 1 - przycisk zwolniony


void odczyt_przyc(void){ //funkcja odczytujca stan przyciskw
//odczyt nastpuje poprzez sprawdzenie stanu "poprzedniego" i "obecnego" - 
//std kopiowanie zawartoci, ktra w "obecnej" iteracji staje si ju "poprzedni"
poprz_przyc1 = akt_przyc1;
poprz_przyc2 = akt_przyc2;
if(PRZYC1){akt_przyc1 = 0;} else {akt_przyc1 = 1;}
if(PRZYC2){akt_przyc2 = 0;} else {akt_przyc2 = 1;}
}


int main(void){

DDRB = 0b00111111; //konfiguracja rejestrw kierunku
DDRC = 0b000000;
DDRD = 0b11111111;

PORTB = 0b11111110; //ustawienie wartoci na wyjciach portw
PORTC = 0b111111;
PORTD = 0b11110000;

_delay_ms(100);

if(JEST_REZ_DOD){ //jeeli wykryto pytk przystawki
tryb_pracy = 1; 
} else { 
tryb_pracy = 0; //jeeli nie wykryto
DDRC |= 0b000111;
PORTC |= 0b111111;
}

REZ; //ustawienie rejestru ADMUX
ADCSR = 0b11100011; //zaczenie ADC ; start Free Running Mode ; wybr trybu Free Running ; brak przerwania ; wyczenie przerwania od ADC ; podzia przez 8


// ***************************************************************
// ********** Tryb bramy jednoskrzydowej ************************
// ***************************************************************


while(tryb_pracy == 0){

odczyt_przyc();
//--------------------
if(poprz_przyc1 == 1 && akt_przyc1 == 0){ //decyzja podejmowana na zboczu opadajcym tu po wciniciu przycisku na pilocie
autozam = 0;
if(otw == 0 && zam == 0){autozam = 1; odliczanie = 0;} //autozamykanie uruchomi si tylko wtedy, gdy brama wczeniej staa
otw++;


if(otw == 1 && zam == 0){ //konfiguracja do otwierania bramy
ZAL_TRAFO;
_delay_ms(OPOZN_ZAL_TRAFA);
WYL_PK1;
ZAL_PK2;
_delay_ms(OPOZN_ROZRUCH);
}

}

//--------------------

if(poprz_przyc2 == 1 && akt_przyc2 == 0){ //decyzja podejmowana na zboczu opadajcym tu po wciniciu przycisku na pilocie
autozam = 0;
zam++;


if(otw == 0 && zam == 1){ //konfiguracja do zamykania bramy
ZAL_TRAFO;
_delay_ms(OPOZN_ZAL_TRAFA);
ZAL_PK1;
WYL_PK2;
_delay_ms(OPOZN_ROZRUCH);
}}

//--------------------

if((otw == 1 && zam == 1) || otw >= 2 || zam >= 2){ //konfiguracja do natychmiastowego zatrzymania bramy: albo nacinicie drugiego przycisku, albo ponowne tego samego
WYL_PK1;
WYL_PK2;
WYL_TRAFO;
zam = 0;
otw = 0;
autozam = 0;
odliczanie = 0;
_delay_ms(500);
}

//--------------------

REZ; //odczyt napicia z rezystora
_delay_ms(2);
wart_rez = (ADCL | (ADCH<<8));


POT_MIN; //odczyt napicia z potencjometru P1 (prd minimalny)
_delay_ms(2);
wart_pot_min = (ADCL | (ADCH<<8));


POT_MAX; //odczyt napicia z potencjometru P2 (prd maksymalny)
_delay_ms(2);
wart_pot_max = (ADCL | (ADCH<<8));

//--------------------

if(wart_rez >= wart_pot_max || wart_rez <= wart_pot_min){ //po wykryciu przecienia lub niedocienia - zatrzymaj 
WYL_PK1;
WYL_PK2;
WYL_TRAFO;
	if(otw == 0 && zam == 1){ //jeeli brama si zamykaa, to wycz samozamykanie
	autozam = 0;
	odliczanie = 0;
	}
zam = 0;
otw = 0;
}

//--------------------

if(autozam == 1 && odliczanie < IL_CYKLI && otw == 0 && zam == 0){ odliczanie++;}

if(autozam == 1 && odliczanie >= IL_CYKLI){ 
ZAL_TRAFO;
_delay_ms(OPOZN_ZAL_TRAFA);
ZAL_PK1;
WYL_PK2;
_delay_ms(OPOZN_ROZRUCH);
autozam = 0;
odliczanie = 0;
otw = 0; //ustawienie flag zamykania - pomocne, jeeli bdzie si chciao zatrzyma ten proces przyciskiem na pilocie
zam = 1;
}
}



// ***************************************************************
// ************ Tryb bramy dwuskrzydowej ************************
// ***************************************************************


while(tryb_pracy == 1){

odczyt_przyc();

//--------------------
if(poprz_przyc1 == 1 && akt_przyc1 == 0){ //decyzja podejmowana na zboczu opadajcym tu po wciniciu przycisku na pilocie
autozam = 0;
if(otw == 0 && zam == 0 && otw_dod == 0 && zam_dod == 0){autozam = 1; odliczanie = 0;} //autozamykanie uruchomi si tylko wtedy, gdy brama wczeniej staa
otw++;
otw_dod++;

if(otw == 1 && zam == 0 && otw_dod == 1 && zam_dod == 0){ //konfiguracja do otwierania bramy
ZAL_TRAFO;
_delay_ms(OPOZN_ZAL_TRAFA);
WYL_PK1;
ZAL_PK2;
_delay_ms(OPOZN_M_SILOWNIKAMI);
WYL_PK1_DOD;
ZAL_PK2_DOD;
_delay_ms(OPOZN_ROZRUCH);
}
}


//--------------------

if(poprz_przyc2 == 1 && akt_przyc2 == 0){ //decyzja podejmowana na zboczu opadajcym tu po wciniciu przycisku na pilocie
autozam = 0;
zam++;
zam_dod++;

if(otw == 0 && zam == 1 && otw_dod == 0 && zam_dod == 1){ //konfiguracja do zamykania bramy
ZAL_TRAFO;
_delay_ms(OPOZN_ZAL_TRAFA);
ZAL_PK1;
WYL_PK2;
_delay_ms(OPOZN_M_SILOWNIKAMI);
ZAL_PK1_DOD;
WYL_PK2_DOD;
_delay_ms(OPOZN_ROZRUCH);
}}

//--------------------

if((otw == 1 && zam == 1) || otw >= 2 || zam >= 2 || (otw_dod == 1 && zam_dod == 1) || otw_dod >= 2 || zam_dod >= 2){ //konfiguracja do natychmiastowego zatrzymania bramy: albo nacinicie drugiego przycisku, albo ponowne tego samego
WYL_PK1;
WYL_PK2;
WYL_PK1_DOD;
WYL_PK2_DOD;
WYL_TRAFO;
zam = 0;
otw = 0;
otw_dod = 0;
zam_dod = 0;
autozam = 0;
odliczanie = 0;
_delay_ms(500);
}

//--------------------

REZ; //odczyt napicia z rezystora
_delay_ms(2);
wart_rez = (ADCL | (ADCH<<8));


POT_MIN; //odczyt napicia z potencjometru P1 (prd minimalny)
_delay_ms(2);
wart_pot_min = (ADCL | (ADCH<<8));


POT_MAX; //odczyt napicia z potencjometru P2 (prd maksymalny)
_delay_ms(2);
wart_pot_max = (ADCL | (ADCH<<8));


REZ_DOD; //odczyt napicia z rezystora
_delay_ms(2);
wart_rez_dod = (ADCL | (ADCH<<8));


POT_MIN_DOD; //odczyt napicia z potencjometru P1 (prd minimalny)
_delay_ms(2);
wart_pot_min_dod = (ADCL | (ADCH<<8));


POT_MAX_DOD; //odczyt napicia z potencjometru P2 (prd maksymalny)
_delay_ms(2);
wart_pot_max_dod = (ADCL | (ADCH<<8));

//--------------------

if(wart_rez >= wart_pot_max || wart_rez <= wart_pot_min){ //po wykryciu przecienia lub niedocienia na jednym skrzydle - zatrzymaj je
WYL_PK1;
WYL_PK2;

if((otw == 0 && zam == 1) || (otw_dod == 0 && zam_dod == 1)){ //jeeli brama si zamykaa, to wycz samozamykanie
	autozam = 0;
	odliczanie = 0;
	}
	
zam = 0;
otw = 0;
}

//--------------------

if(wart_rez_dod >= wart_pot_max_dod || wart_rez_dod <= wart_pot_min_dod){ //po wykryciu przecienia lub niedocienia na drugim skrzydle - zatrzymaj je
WYL_PK1_DOD;
WYL_PK2_DOD;

if((otw == 0 && zam == 1) || (otw_dod == 0 && zam_dod == 1)){ //jeeli brama si zamykaa, to wycz samozamykanie
	autozam = 0;
	odliczanie = 0;
	}
	
zam_dod = 0;
otw_dod = 0;
}

//--------------------

if(otw == 0 && zam == 0 && otw_dod == 0 && zam_dod == 0 ){ //wyczenie transformatora, jeeli obydwa skrzyda stoj
WYL_TRAFO;
}

//--------------------

if(autozam == 1 && odliczanie < IL_CYKLI_DOD && otw == 0 && zam == 0 && otw_dod == 0 && zam_dod == 0){ odliczanie++;}

if(autozam == 1 && odliczanie >= IL_CYKLI_DOD){ 
ZAL_TRAFO;
_delay_ms(OPOZN_ZAL_TRAFA);
ZAL_PK1;
WYL_PK2;
_delay_ms(OPOZN_M_SILOWNIKAMI);
ZAL_PK1_DOD;
WYL_PK2_DOD;
_delay_ms(OPOZN_ROZRUCH);
autozam = 0;
odliczanie = 0;
otw = 0; //ustawienie flag zamykania - pomocne, jeeli bdzie si chciao zatrzyma ten proces przyciskiem na pilocie
zam = 1;
otw_dod = 0;
zam_dod = 1;
}


}}